"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mongoDbDriverFactory = exports.MongodbDriver = exports.using = void 0;
const mongodb_1 = require("mongodb");
const undef_check_1 = require("undef-check");
const random_1 = require("@jellybeanci/random");
const events_1 = require("events");
var using_statement_1 = require("using-statement");
Object.defineProperty(exports, "using", { enumerable: true, get: function () { return using_statement_1.using; } });
class MongodbDriver extends events_1.EventEmitter {
    id;
    alive = false;
    client;
    database;
    collection;
    constructor(uri) {
        super();
        this.emit("beforeCreate");
        (0, undef_check_1.undefCheck)(uri, "URI cannot be passed Empty or Undefined.");
        this.client = new mongodb_1.MongoClient(uri);
        this.emit("created");
    }
    async initialize() {
        if (this.alive)
            return;
        this.emit("beforeInitialized");
        await this.client.connect();
        this.alive = true;
        this.id = `id:${(0, random_1.randomInt)(10_000)}`;
        this.emit("initialized");
    }
    openDatabase(databaseName) {
        (0, undef_check_1.undefCheck)(databaseName, "Database Name cannot be passed Empty or Undefined.");
        this.checkAlive();
        this.database = this.client.db(databaseName);
        return this;
    }
    db(databaseName) {
        return this.openDatabase(databaseName);
    }
    openConnection(collectionName) {
        (0, undef_check_1.undefCheck)(collectionName, "Collection Name cannot be passed Empty or Undefined.");
        this.checkAlive();
        this.collection = this.database.collection(collectionName);
        return this.collection;
    }
    get(collectionName) {
        return this.openConnection(collectionName);
    }
    access() {
        this.checkAlive();
        return this.collection;
    }
    col() {
        return this.access();
    }
    isAlive() {
        return this.alive;
    }
    checkAlive() {
        if (!this.isAlive())
            throw Error("Connection is not alive. Connection dead or never initialized.");
    }
    async autoClose() {
        this.emit("beforeClose");
        if (!this.isAlive())
            return false;
        await this.client.close();
        this.emit("closed");
        return !(this.alive = false);
    }
    async close() {
        return await this.autoClose();
    }
    dispose() {
        this.emit("disposed");
        return Promise.resolve(void this.autoClose());
    }
    static async create(uri, alive = true) {
        const mongoDriver = new MongodbDriver(uri);
        if (alive)
            await mongoDriver.initialize();
        return mongoDriver;
    }
}
exports.MongodbDriver = MongodbDriver;
async function mongoDbDriverFactory(uri, alive = true) {
    return MongodbDriver.create(uri, alive);
}
exports.mongoDbDriverFactory = mongoDbDriverFactory;
